/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * Should you need to contact me, the author, you can do so either by
 * e-mail - mail your message to <vojtech@ucw.cz>, or by paper mail:
 * Vojtech Pavlik, Simunkova 1594, Prague 8, 182 00 Czech Republic
 */

/* Program based on EvTest.c 
 * Modified by David Martinez Oliveira (picoflamingo AT papermint-designs.com)
 * for the
 * AAXH tutorial: http://papermint-designs.com/community/taxonomy/term/92
 * 
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <fcntl.h>
#include <unistd.h>

#include <linux/input.h>


#define SIZE 3
#define SIZE2 SIZE * SIZE

static char _pat[SIZE2];

/* XXX: To know how to get this information check Evtest.c */
static int _width = 1080;
static int _height = 1920;

typedef struct pat_t
{
  char *cmd;
  int pat[SIZE2];

} PAT;

/* Gesture <-> Command  Mapping */
static PAT pat_cmd[] = {
  {"am start -a android.media.action.IMAGE_CAPTURE",
   {0, 0, 0,
    0, 0, 0,
    1, 2, 3} 
  },
  {"am start http://papermint-designs.com/community", 
   {0, 3, 0,
    0, 2, 0,
    0, 1, 0}
  },
  /* XXX: This marks the end of the array. Do not delete */
  {NULL, 
   {-1,-1,-1,
    -1,-1,-1,
    -1,-1,-1}
  }
};

#if 0
/* In case you want to try... */
static PAT pat_cmd[] = {
  {"/data/local/tmp/hellow &",
   {0, 0, 0,
    0, 0, 0,
    1, 2, 3} 
  },
  {"kill $(pidof hellow)", 
   {0, 3, 0,
    0, 2, 0,
    0, 1, 0}
  },
  /* XXX: This marks the end of the array. Do not delete */
  {NULL, 
   {-1,-1,-1,
    -1,-1,-1,
    -1,-1,-1}
  }
};
#endif


void
match_pat ()
{
  int  i, j;
  int  found;

  for (i = 0; pat_cmd[i].cmd; i++, found = 0)
    {

      for (j = 0; j < SIZE2; j++) 
	{
	  found = 0;

	  if (_pat[j] != pat_cmd[i].pat[j]) break;
	  found = 1;
	}
      if (found)
	{
	  printf ("Pattern %i found... running command '%s'\n", i, pat_cmd[i].cmd);
	  printf ("System Result: %x\n", system (pat_cmd[i].cmd));
	  perror ("system:");
	  return;
	}
    }
  return;
}

void
print_pat ()
{
  int i, j;

  for (i = 0; i < SIZE; i++)
    {
      for (j = 0; j < SIZE; j++)
	printf ("%d ", _pat[j + i *SIZE]);
      printf ("\n");
    }
	      
}

void
reset_pat ()
{
  int i;

  for (i = 0; i < SIZE2; i++) _pat[i] = 0;
  
}

int 
main (int argc, char **argv)
{
  int    fd, rd, i;
  int    x, y, x0, y0, cont;
  char   name[256] = "Unknown";
  struct input_event ev[64];

  if (argc < 2) 
    {
      printf ("Usage: touch /dev/input/eventX\n");
      printf ("Where X = input device number.\nTry getevents to find the device\n");
      return 1;
    }
  
  /* Open the input device */
  if ((fd = open (argv[argc - 1], O_RDONLY)) < 0) 
    {
      perror ("touch:");
      return 1;
    }
    
  /* Print the name of the input device for user confirmation */
  ioctl(fd, EVIOCGNAME(sizeof(name)), name);
  printf("Input device name: \"%s\"\n", name);
  
  x0 = y0 = x = y = -1;
  cont = -1;
  
  while (1) 
    {
      rd = read (fd, ev, sizeof(struct input_event) * 64);
    
      if (rd < (int) sizeof(struct input_event)) 
	{
	  printf("ERROR: Cannot read events\n");
	  perror("\n(touch) error reading:");
	  return 1;
	}

      for (i = 0; i < rd / sizeof(struct input_event); i++)
	{
	  switch (ev[i].type)
	    {
	    case 3: /* Coordinates and tap counter */
	      {
		if (ev[i].code == 57) /* Tap counter */
		  {
		    if (ev[i].value == -1) /* If Tap finished */
		      {
			printf ("Gesture Finished\n");
			print_pat ();
			match_pat ();
			
			reset_pat ();
			x = y = x0 = y0 = cont = -1;
		      }
		    else /* else we had just tapped the screen */
		      {
			printf ("Gesture Started\n");
			cont ++;
		      }
		  }
		/* Process tap coordinates */
		if (ev[i].code == 54)
		  {
		    y = (int) ((float)SIZE * (float)ev[i].value / (float)_height);
		  }
		else if (ev[i].code == 53)
		  {
		    x = (int) ((float)SIZE * (float)ev[i].value / (float)_width);
		  }

		if ((x == -1) || (y == -1)) continue;
		if ((x != x0) || (y != y0))
		  {
		    x0 = x;
		    y0 = y;
		    cont++;
		    _pat[x0 + SIZE * y0] = cont;
		    printf ("(%d,%d) = %d\n", x, y, cont);
		  }
	      }
	    default:
	      {
		/* Event not processed.*/
		break;
	      }
	    }
	}
    }

  return 0;
}

//  arm-linux-androideabi-gcc -o touch touch.c
